#
# testEngine.py - unit tests for Engine.py
#

import unittest
import weblib

class EngineTestCase(unittest.TestCase):


    def check_request(self):
        import os
        os.environ["QUERY_STRING"]="enginetest"
        engine = weblib.Engine()
        
        assert engine.request.querystring=="enginetest", \
               "engine has wrong default request"

        del engine

        req = weblib.Request(querystring="e=mc2&this=a+test")
        engine = weblib.Engine(request=req)
        assert engine.request.querystring=="e=mc2&this=a+test", \
               "engine has wrong passed-in request:" + \
               engine.request.querystring


    def check_form(self):

        weblib.MYFORM = {"a":"bcd"}

        myscript = "import weblib\n" +\
                   "assert weblib.request.form is weblib.MYFORM, 'request uses wrong form'"

        req = weblib.Request(form=weblib.MYFORM)
        weblib.Engine(request=req, script=myscript).run()


        del weblib.MYFORM


    def check_environ(self):
        myenv = {"A":"B"}
        eng = weblib.Engine(request=weblib.Request(environ=myenv))
        assert eng.request["A"] == "B", "engine has wrong passed-in environ"



    def check_simple(self):
        engine = weblib.Engine(script="print 'hello'")
        engine.run()
        assert engine.response.buffer == "hello\n", \
               "engine doesn't execute the script!"
        
        

    def check_responseEnd(self):
        res = 1
        try:
            engine = weblib.Engine(script="import weblib; weblib.response.end()")
            engine.run()
        except SystemExit:
            res = 0

        assert res==1, "Engine doesn't trap system.exit()!"
